/* Program resamp.cc    */
/* By Harald Stauss     */
/* Last change 22.08.97 */

#include <ctype.h>
#include <math.h>
#include "tsa.h"
#include "../nrc/nrutil.h" 
#include "../nrc/nrutil.cc"
#include "../nrc/four1.cc"
#include "../nrc/moment.cc"

main(int argc, char *argv[])
{
           short    i;
           int      res;
  unsigned long     fn,n,eospec,nout;
           float    ave, adev, sdev, var, skew, curt;
           float    *data;
           float    sr,fifreq;
           char     *fin,*fout,*ftin,*ftout;
           char     *fpin,*fpout,*fptin,*fptout;
           REGIST   r,f;


  /* H e l l o   w o r l d */
  cout << "RESAMP Ver. 1.1 by H. Stauss, 1997 \n";

  /* A r r a y s   a l l o c i e r e n */
  fin=(char *)malloc(255);
  fout=(char *)malloc(255);
  ftin=(char *)malloc(255); fptin=ftin;
  ftout=(char *)malloc(255); fptout=ftout;
  data=(float *)malloc(sizeof(float));

  /* P a r a m e t e r   e i n l e s e n */
  if (argc != 6 && argc!=7)
    {
      cout << "Number of parameters wrong! \n";
      cout << "resamp f_in f_out n f rn [sr] \n";
      cout << "removes a linear trend, \n";
      cout << "zero pads to n values if necessary, \n";
      cout << "calculates the FFT and cuts the spectrum at frequency f, \n";
      cout << "performs a reverse FFT and resamples every rn value. \n";
      cout << "f_in  : name of input file (*.TSA, *.ASC). \n";
      cout << "f_out : name of output file (*.TSA, *.ASC). \n";
      cout << "sr    : sampling rate (optional ASCII files) \n";
      goto ende;
    }

  strcpy(fin,argv[1]); fpin=fin;
  strcpy(fout,argv[2]); fpout=fout;
  fptin=strrchr(fpin,'.');
  fptout=strrchr(fpout,'.');

  strcpy(ftin,fptin); fptin=ftin; 
  strcpy(ftout,fptout); fptout=ftout;
  for (i=0; fptin[i]; i++) fptin[i]=toupper(fptin[i]);
  for (i=0; fptout[i]; i++) fptout[i]=toupper(fptout[i]);

  sscanf(argv[3],"%u",&fn);
  sscanf(argv[4],"%f",&fifreq);
  sscanf(argv[5],"%d",&res); 
  if (argc==7) sscanf(argv[6],"%f",&sr);
  if ((strcmp(fptin,".ASC")==0) && (argc!=7)) {
    cout << "Need sampling rate for ASCII Files!\n";
    goto ende;
  }
  nout=fn/res;

  cout << "Input file        : " << fin << "\n";
  cout << "Output file       : " << fout << "\n";

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(fptin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(fptin,".ASC")==0) r.ReadASCII(fin,sr);
  if ((strcmp(fptin,".TSA")!=0) && (strcmp(fptin,".ASC")!=0)) {
    cout << "File type for input not recognized! \n";
    goto ende;
  }
  cout << "Number of channels: " << r.n << "\n";

  sr=r.ch[0].sr;
  for (i=1; i<r.n; i++)
    if (r.ch[i].sr!=sr) {
      cout << "sampling rate must be equal in all channels! \n";
      goto ende;
    }
  for (i=0; i<r.n; i++) f.AddChannel(nout, sr/res,
				     r.ch[i].min, r.ch[i].max, 0, 0,
				     r.ch[i].name, r.ch[i].unit);
  data=(float *)realloc(data,2*fn*sizeof(float));

  /* M A I N L O O P  */
  for (i=0; i < r.n; i++) {
    cout << "n in channel " << i+1 << ": " << r.ch[i].n <<
      " FFT based on " << fn << " values ";
    if (fn<=r.ch[i].n) cout << "\n"; else cout << "=> zero padded! \n";
    // Mittelwert berechnen;
    n=fn;
    if (r.ch[i].n<fn) n=r.ch[i].n;
    moment(r.ch[i].data-1,n,&ave,&adev,&sdev,&var,&skew,&curt);
    /* Daten fuer FFT kopieren */
    for (n=0; n<fn; n++) {
      if (n<r.ch[i].n) { 
	data[n*2]=r.ch[i].data[n]-ave;
	data[n*2+1]=0;
      }
      else {
	data[n*2]=0;
	data[n*2+1]=0;
      }
    }
    four1(data-1,fn,1);
    // Spectrum abschneiden;
    data[0]=0; data[1]=0;
    eospec=(unsigned long)(fn*fifreq/sr);
    for (n=eospec; n<=fn-eospec; n++) {
      data[n*2]=0;
      data[n*2+1]=0;
    }
    // Ruecktransformieren;
    four1(data-1,fn,-1);
    // in neue REGIST copieren;
    for(n=0; n<nout; n++) f.ch[i].data[n]=data[2*n*res]/fn+ave;
  } /* endfor */

  /* D a t e i   s c h r e i b e n */
  if (strcmp(fptout,".TSA")==0) f.WriteTSA(fout);
  if (strcmp(fptout,".ASC")==0) f.WriteASCII(fout);
  if ((strcmp(fptout,".TSA")!=0) && (strcmp(fptout,".ASC")!=0)) {
    printf("File type for output not recognized! \n");
    goto ende;
  }

 ende:
  free(data);free(fin);free(ftin);free(fout);free(ftout);
  return(0);
} /* Ende von main */
